/*
 * @(#) ImageDataViewer.java 1.0 99/10/4
 * Copyright (c) 1999 Larry Rodrigues
 */
package app;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.color.*;
import java.beans.*;
import com.vistech.util.*;
import vis.beans.plotter.*;
import com.vistech.imagedata.*;
import java.util.*;

import com.sun.image.codec.jpeg.*;
 /** An application that displays image information.
   * @version 1.0  10 April 1999
   * @author Lawrence Rodrigues
   */
public class ImageDataViewer extends JPanel {
   ImagePanel viewer;
   BufferedImage image = null;

   JTable imageInfoTable;
   JTable imageStatsTable;

   int imageWidth, imageHeight;
   JFrame plotFrame, histoFrame;

   public static BufferedImage readAsBufferedImage(String filename) {
      try {
         FileInputStream fis = new FileInputStream(filename);
         JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(fis);
         BufferedImage bi = decoder.decodeAsBufferedImage();
         return bi;
      } catch(Exception e) {
         System.out.println(e);
         return null;
      }
   }

   public void displayStats(ColorModel cm, SampleModel sm, DataBuffer db){
       ColorSpace cs = cm.getColorSpace();
       int numComponents = sm.getNumBands();
       String name[] = new String[numComponents];
       System.out.println("num components = "+ numComponents);

       for(int i=0;i<numComponents;i++){
           name[i] = cs.getName(i);
           System.out.println("name = "+ name[i]);
       }
       int[][] imstats = ImageInfoUtil.getImageStats(sm, db, new Dimension(imageWidth, imageHeight));
       String[] colnames = new String[] {"Sample", "Min", "Max", "Mean"};
       String rowdata[][] = new String[imstats.length][colnames.length+1];

       for(int i=0; i<imstats.length;i++){
           rowdata[i][0] = name[i];
           //columns
           for(int j=0;j<imstats[0].length;j++){
               rowdata[i][j+1] = Integer.toString(imstats[i][j]);
           }
       }
       imageStatsTable = new JTable(rowdata, colnames);
   }

   public void displayImageInfo(BufferedImage img){
      String prop[] = img.getPropertyNames();
      if(prop != null) {
         for(int i=0;i< prop.length; i++) {
             System.out.println(prop[i] + "   "+ img.getProperty(prop[i]));
         }
      }
      SampleModel sm = img.getSampleModel();
      WritableRaster wr = img.getRaster();
      DataBuffer db = wr.getDataBuffer();
      ColorModel cm = img.getColorModel();
      String rowdata[][] = ImageInfoUtil.getImageInfo(img);
      if(rowdata == null) return;
      String colnames[] = {"Properties ", "value"};
      imageInfoTable = new JTable(rowdata, colnames);
      imageWidth = img.getWidth();
      imageHeight = img.getHeight();
      displayStats(cm,sm,db);
   }

   public void displayHistogram(BufferedImage img) {
      if(histoFrame != null){
         histoFrame.show();
         return;
      }
      try {
         int[][] pix = ImageInfoUtil.getPixelSamples(img);
         ClassLoader cl = (MultiHistogram.class).getClassLoader();
         MultiHistogram histo = (MultiHistogram)Beans.instantiate(cl, "vis.beans.plotter.MultiHistogram");
         histo.setSize(400,300);
         histoFrame = new JFrame();
         histoFrame.addWindowListener(
         new WindowAdapter() {
            public void windowClosing(WindowEvent e){
               histoFrame.hide();
            }
         } );
         histoFrame.setTitle("Histogram");
         Container cp = histoFrame.getContentPane();
         histoFrame.getContentPane().setLayout(new GridLayout(1,1));
         cp.add(histo);
         histoFrame.pack();
         histoFrame.setSize(new Dimension(400, 300));
         histoFrame.show();
         String[] colstr = {"red sample", "green sample", "blue sample", "yellow"};
         Color[]  col = {Color.red, Color.green, Color.blue, Color.yellow};
         histo.setLegendOn(true);
         for (int i=0; i<pix.length;i++) {
           histo.addGraph(null, pix[i]);
           histo.addLegend(col[i], colstr[i]);
         }
         histo.setNumBars(6);
         histo.setTitleString("Histogram ");
         histo.setXLabelString("Pixel sample values");
         histo.setYLabelString("Frequency");
         histo.repaint();
      }catch (Exception e) {
         System.out.println(e);
      }
   }

    public void displayPlot(BufferedImage img) {
      if(plotFrame != null){
         plotFrame.show();
         return;
      }
      try {
      int[][] pix = ImageInfoUtil.getPixelSamples(img);
      ClassLoader cl = (MultiXYPlot.class).getClassLoader();
      MultiXYPlot xyp = (MultiXYPlot)Beans.instantiate(cl, "vis.beans.plotter.MultiXYPlot");
      plotFrame = new JFrame();
      plotFrame.addWindowListener(
         new WindowAdapter() {
            public void windowClosing(WindowEvent e){
               plotFrame.hide();
            }
         } );
      plotFrame.setTitle("XY Plot");
      Container cp = plotFrame.getContentPane();
      plotFrame.getContentPane().setLayout(new GridLayout(1,1));
      cp.add(xyp);
      plotFrame.pack();
      plotFrame.setSize(new Dimension(400, 300));
      plotFrame.show();
      String[] colstr = {"red sample", "green sample", "blue sample", "yellow"};
      Color[]  col = {Color.red, Color.green, Color.blue, Color.yellow};
      xyp.setLegendOn(true);
      int binWidth = 20;
      int numBins = 16;

      int[] x = new int[numBins];
      for (int i=0;i< numBins;i++)   x[i] = i*binWidth;
      int[] y = new int[numBins];
      for(int i=0; i<pix.length;i++){
           int k = 0;
           for (int j=0; j < pix[0].length;j++) {
                k = pix[i][j] /binWidth;
                if(k >= numBins) y[numBins-1]++;
                y[k]++;
           }
           xyp.addGraph(x, y);
           xyp.addLegend(col[i], colstr[i]);
           y = new int[numBins];
      }
      xyp.setTitleString(" ");
      xyp.setXLabelString("Sample Values");
      xyp.setYLabelString("Frequency");
      xyp.repaint();
      }catch (Exception e) {
          System.out.println(e);
      }
   }

   public void loadAndDisplay(String filename){
      image = readAsBufferedImage(filename);
      BufferedImage img = convertPackedToInterleaved(image);
      displayImage(img);
   }

   public void displayImage(BufferedImage img) {
      image = img;
      int width = img.getWidth();
      int height = img.getHeight();
      displayImageInfo(img);
      displayImage(img, width, height);
   }

   public void displayImage(BufferedImage img, int width, int height) {
      JFrame fr = new JFrame();
      fr.addWindowListener(
         new WindowAdapter() {
            public void windowClosing(WindowEvent e){
               System.exit(0);
            }
         } );
      fr.setTitle("Image Data Viewer");
      viewer = new ImagePanel(img);
      viewer.addMouseListener(new LaunchMenuAdapter());
      JScrollPane infojs = new JScrollPane(imageInfoTable);
      infojs.setBorder(BorderFactory.createTitledBorder("Image Info"));
      JScrollPane statsjs = new JScrollPane(imageStatsTable);
      statsjs.setBorder(BorderFactory.createTitledBorder("Image Statistics"));
      JSplitPane vsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, statsjs,infojs);
      vsp.setDividerLocation((int)(height/4));
      viewer.setPreferredSize(new Dimension(width, height));
      JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, viewer, vsp);
      Container cp = fr.getContentPane();
      fr.getContentPane().setLayout(new GridLayout(1,1));
      cp.add(sp);
      fr.pack();
      fr.setSize(new Dimension(width+ (int)(width/2), height));
      fr.show();
      sp.setDividerLocation(0.6);
      viewer.repaint();
   }

   protected void popupMenu(JComponent comp,int x, int y){
      JPopupMenu jp = new JPopupMenu("");
      jp.setLightWeightPopupEnabled(true);
      comp.add(jp);
      JMenuItem histo = new JMenuItem("Histogram");
      histo.addActionListener(
         new ActionListener(){
           public void actionPerformed(ActionEvent e){
              if(image != null)displayHistogram(image);
           }
         }
      );
      JMenuItem plot = new JMenuItem("XY Plot");
      plot.addActionListener(
         new ActionListener(){
           public void actionPerformed(ActionEvent e){
              if(image != null) displayPlot(image);
           }
         }
      );

      jp.add(histo);
      jp.add(plot);
      jp.addSeparator();
      jp.show(comp,x,y);
   }

   class LaunchMenuAdapter extends java.awt.event.MouseAdapter {
       public void mousePressed(MouseEvent e) {
         if(SwingUtilities.isRightMouseButton(e)){
            popupMenu((JComponent)e.getSource(), e.getX(), e.getY());
        }
   }
  }

   class ImagePanel extends JComponent {
      BufferedImage image;
      public ImagePanel(BufferedImage img){
         image = img;
      }

      public void paintComponent(Graphics g){
         Rectangle rect = this.getBounds();
         if(image != null) {
            g.drawImage(image, 0,0,rect.width, rect.height, null);
        }
      }
   }

   public static BufferedImage createImage(int imageWidth, int imageHeight, int imageDepth, byte data[]){
      ComponentColorModel ccm = new ComponentColorModel(
                          ColorSpace.getInstance(ColorSpace.CS_GRAY),
                          new int[] {imageDepth}, false, false,
                          Transparency.OPAQUE,
                          DataBuffer.TYPE_BYTE);
      ComponentSampleModel csm = new ComponentSampleModel(
                                    DataBuffer.TYPE_BYTE,
                                    imageWidth, imageHeight,1,imageWidth, new int[] {0});
      DataBuffer dataBuf = new DataBufferByte((byte[])data, imageWidth*imageHeight);
      WritableRaster wr = Raster.createWritableRaster(csm, dataBuf, new Point(0,0));
      Hashtable ht = new Hashtable();
      ht.put("owner", "Lawrence Rodrigues");
      return  new BufferedImage(ccm, wr, false, ht);
   }

   public static BufferedImage createBandedImage(int imageWidth, int imageHeight, int imageDepth, short data[][]){
      ComponentColorModel ccm = new ComponentColorModel(
                          ColorSpace.getInstance(ColorSpace.CS_sRGB),
                          new int[] {8,8,8}, false, false,
                          Transparency.OPAQUE,
                          DataBuffer.TYPE_USHORT);
      BandedSampleModel csm = new BandedSampleModel(
                                    DataBuffer.TYPE_USHORT,
                                    imageWidth, imageHeight, imageWidth,
                                    new int[] {0,1,2},new int[] {0,1,2} );
      DataBuffer dataBuf = new DataBufferUShort(data, imageWidth*imageHeight);
      WritableRaster wr = Raster.createWritableRaster(csm, dataBuf, new Point(0,0));
      Hashtable ht = new Hashtable();
      ht.put("owner", "Lawrence Rodrigues");
      return  new BufferedImage(ccm, wr, false, ht);
   }

    public static BufferedImage createInterleavedImage1(int imageWidth, int imageHeight,
                int imageDepth, short data[]){
      ComponentColorModel ccm = new ComponentColorModel(
                          ColorSpace.getInstance(ColorSpace.CS_sRGB),
                          new int[] {8,8,8,8}, true, false,
                          Transparency.OPAQUE,
                          DataBuffer.TYPE_USHORT);
      PixelInterleavedSampleModel csm = new PixelInterleavedSampleModel(
                                    DataBuffer.TYPE_USHORT,
                                    imageWidth, imageHeight, 4, imageWidth*4, new int[] {0,1,2,3} );
       DataBuffer dataBuf = new DataBufferUShort(data, imageWidth*imageHeight*4);
      WritableRaster wr = Raster.createWritableRaster(csm, dataBuf, new Point(0,0));
      Hashtable ht = new Hashtable();
      ht.put("owner", "Lawrence Rodrigues");
      return  new BufferedImage(ccm, wr, false, ht);
   }

   public static BufferedImage createInterleavedImage(int imageWidth, int imageHeight,
                int imageDepth, short data[]){
      ComponentColorModel ccm = new ComponentColorModel(
                          ColorSpace.getInstance(ColorSpace.CS_sRGB),
                          new int[] {8,8,8}, false, false,
                          Transparency.OPAQUE,
                          DataBuffer.TYPE_USHORT);
      PixelInterleavedSampleModel csm = new PixelInterleavedSampleModel(
                                    DataBuffer.TYPE_USHORT,
                                    imageWidth, imageHeight, 3, imageWidth*3, new int[] {0,1,2} );
      DataBuffer dataBuf = new DataBufferUShort(data, imageWidth*imageHeight);
      WritableRaster wr = Raster.createWritableRaster(csm, dataBuf, new Point(0,0));

      Hashtable ht = new Hashtable();
      ht.put("owner", "Lawrence Rodrigues");
      return  new BufferedImage(ccm, wr, false, ht);
   }


  public BufferedImage getGrayscaleImage(){
     int wid = 64, ht =64;
      byte data[] = new byte[wid*ht];
      for(int i=0; i<ht; i++){
          for(int j=0; j<wid; j++){
              data[i*wid+j] = (byte)(255* Math.abs(Math.random()));
          }
      }
      return createImage(64,64,8,data);
   }

   public BufferedImage getBandedImage(){
      int wid = 64, ht =64;
      short data1[][] = new short[3][wid*ht];
      for(int k=0;k<3;k++) {
      for(int i=0; i<ht; i++){
          for(int j=0; j<wid; j++){
              data1[k][i*wid+j] = (short)(j*k);
          }
       }
      }
      return createBandedImage(wid,ht,8,data1);
  }

  public BufferedImage convertPackedToInterleaved(BufferedImage img) {
     ColorModel cm = img.getColorModel();
     if(! (cm instanceof DirectColorModel)) return null;
     int wid = img.getWidth(), ht = img.getHeight();
     short data1[] = new short[wid*ht*3];
     SampleModel sm = img.getSampleModel();
     WritableRaster wr = img.getRaster();
     DataBuffer db = wr.getDataBuffer();
     int numbands = wr.getNumBands();
     int sample[] = new int[numbands];
     for(int i=0; i<ht; i++){
          for(int j=0; j<wid; j++){
              int pix[] = null;
              sample = (sm.getPixel(j,i, pix, db));
              for(int l =0 ; l<numbands;l++)
                  data1[i*wid*3+(j*3)+l] =  (short)sample[l];
          }
       }
      return ImageFactory.createInterleavedRGBImage(wid,ht,8,data1, false);
 }

   public static void  main(String[] args){
      ImageDataViewer ip = new ImageDataViewer();
       if(args.length <1) {
         System.out.println("Enter a valid JPEG file name");
         System.exit(0);
      } else ip.loadAndDisplay(args[0]);
   }
}
